@@ -49,12 +49,18 @@ import itertools
 import src.lib.i18n as i18n
 import os
 
+try:
+    import xlrd
+except ImportError:
+    xlrd = None
+
 import wx
 
 from src.config import config
 from src.sysvars import get_default_font, is_gtk
-
 from src.gui._grid_table import GridTable
+from src.interfaces.pys import Pys
+from src.interfaces.xls import Xls
 
 try:
     from src.lib.gpg import sign, verify
@@ -282,60 +288,56 @@ class FileActions(Actions):
         """
 
         filepath = event.attr["filepath"]
+        try:
+            filetype = event.attr["filetype"]
 
-        # Content parsers
+        except KeyError:
+            filetype = "pys"
 
-        dict_grid = self.code_array.dict_grid
+        type2opener = {"pys": (Bz2AOpen, [filepath, "r"],
+                               {"main_window": self.main_window})}
+        if xlrd is not None:
+            type2opener["xls"] = \
+                (xlrd.open_workbook, [filepath], {"formatting_info": True})
 
-        section_readers = {
-            "[shape]": dict_grid.parse_to_shape,
-            "[grid]": dict_grid.parse_to_grid,
-            "[attributes]": dict_grid.parse_to_attribute,
-            "[row_heights]": dict_grid.parse_to_height,
-            "[col_widths]": dict_grid.parse_to_width,
-            "[macros]": dict_grid.parse_to_macro,
+        type2interface = {
+            "pys": Pys,
+            "xls": Xls,
         }
 
-        # Set states for file open
+        # Specify the interface that shall be used
 
+        opener, op_args, op_kwargs = type2opener[filetype]
+        Interface = type2interface[filetype]
+
+        # Set state for file open
         self.opening = True
 
         try:
-            with Bz2AOpen(filepath, "r", main_window=self.main_window) \
-                    as infile:
+            with opener(*op_args, **op_kwargs) as infile:
                 # Make loading safe
                 self.approve(filepath)
 
-                # Abort if file version not supported
-
-                version = self._get_file_version(infile)
-
-                if version not in self.pys_versions:
-                    text = _("File version {version} unsupported (not 0.1).")
-                    text = text.format(version=version,
-                                       allver=self.pys_versions)
-                    post_command_event(self.main_window, self.StatusBarMsg,
-                                       text=text)
-                    return False
-
                 # Disable undo
                 self.grid.code_array.unredo.active = True
 
-                for line in infile:
-                    stripped_line = line.decode("utf-8").strip()
-                    if stripped_line:
-                        # There is content in this line
-                        if stripped_line in section_readers:
-                            # Switch parser
-                            parser = section_readers[stripped_line]
-                        else:
-                            # Parse line
-                            parser(line)
-                            if parser == dict_grid.parse_to_shape:
-                                # Empty grid
-                                self.clear(self.code_array.shape)
+                try:
+                    wx.BeginBusyCursor()
+                    self.grid.Disable()
+                    self.clear()
+                    interface = Interface(self.grid.code_array, infile)
+                    interface.to_code_array()
 
-                                self.grid.GetTable().ResetView()
+                except ValueError, err:
+                    post_command_event(self.main_window, self.StatusBarMsg,
+                                       text=str(err))
+
+                finally:
+                    self.grid.GetTable().ResetView()
+                    post_command_event(self.main_window, self.ResizeGridMsg,
+                                       shape=self.grid.code_array.shape)
+                    self.grid.Enable()
+                    wx.EndBusyCursor()
 
                 # Execute macros
                 self.main_window.actions.execute_macros()
@@ -360,6 +362,7 @@ class FileActions(Actions):
             pass
 
         finally:
+            # Unset state for file open
             self.opening = False
 
     def sign_file(self, filepath):
@@ -411,84 +414,51 @@ class FileActions(Actions):
 
         filepath = event.attr["filepath"]
 
-        dict_grid = self.code_array.dict_grid
-
         io_error_text = _("Error writing to file {filepath}.")
         io_error_text = io_error_text.format(filepath=filepath)
 
-        # The output generators yield the lines for the outfile
-        output_generators = [
-            # Grid content
-            dict_grid.grid_to_strings,
-            # Cell attributes
-            dict_grid.attributes_to_strings,
-            # Row heights
-            dict_grid.heights_to_strings,
-            # Column widths
-            dict_grid.widths_to_strings,
-            # Macros
-            dict_grid.macros_to_strings,
-        ]
-
         # Set state to file saving
         self.saving = True
 
         # Make sure that old save file does not get lost on abort save
-        try:
-            tmpfile = filepath + "~"
-            os.rename(filepath, tmpfile)
-
-        except OSError:
-            # No file present
-            pass
+        tmpfile = filepath + "~"
 
         try:
             wx.BeginBusyCursor()
-            with Bz2AOpen(filepath, "wb", main_window=self.main_window) \
+            self.grid.Disable()
+            with Bz2AOpen(tmpfile, "wb", main_window=self.main_window) \
                     as outfile:
 
-                # Header
-                outfile.write("[Pyspread save file version]\n")
-                outfile.write("{ver}\n".format(ver=self.pys_versions[-1]))
+                try:
+                    pys = Pys(self.grid.code_array, outfile)
+                    pys.from_code_array()
 
-                # Save cycle
+                except ValueError, err:
+                    post_command_event(self.main_window, self.StatusBarMsg,
+                                       text=err)
 
-                for generator in output_generators:
-                    for line in generator():
-                        outfile.write(line.encode("utf-8"))
-                        if outfile.aborted:
-                            break
-                    if outfile.aborted:
-                            break
-
-        except IOError:
-            msg = _("Error writing to file {filepath}.")
-            msg = msg.format(filepath=filepath)
+            # Move save file from temp file to filepath
             try:
-                post_command_event(self.main_window, self.StatusBarMsg,
-                                   text=msg)
-            except TypeError:
-                # The main window does not exist any more
-                pass
+                os.rename(tmpfile, filepath)
 
-            wx.EndBusyCursor()
-
-            return False
-
-        finally:
-            # Remove partial save file if save has been aborted
-
-            try:
-                if outfile.aborted:
-                    os.rename(tmpfile, filepath)
-                else:
-                    os.remove(tmpfile)
             except OSError:
                 # No tmp file present
                 pass
 
+        except IOError:
+            try:
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=io_error_text)
+            except TypeError:
+                # The main window does not exist any more
+                pass
+
+            return False
+
+        finally:
             self.saving = False
             wx.EndBusyCursor()
+            self.grid.Enable()
 
         # Mark content as unchanged
         try:
@@ -1023,6 +993,20 @@ class GridActions(Actions):
     def on_mouse_over(self, key):
         """Displays cell code of cell key in status bar"""
 
+        def split_lines(string, line_length=80):
+            """Returns string that is split into lines of length line_length"""
+
+            result = u""
+            line = 0
+
+            while len(string) > line_length * line:
+                line_start = line * line_length
+                result += string[line_start:line_start+line_length]
+                result += '\n'
+                line += 1
+
+            return result[:-1]
+
         row, col, tab = key
 
         if (row, col) != self.prev_rowcol and row >= 0 and col >= 0:
@@ -1038,6 +1022,22 @@ class GridActions(Actions):
             post_command_event(self.main_window, self.StatusBarMsg,
                                text=hinttext)
 
+            cell_res = self.grid.code_array[row, col, tab]
+
+            if cell_res is None:
+                self.grid.SetToolTip(None)
+                return
+
+            try:
+                cell_res_str = unicode(cell_res)
+            except UnicodeEncodeError:
+                cell_res_str = unicode(cell_res, encoding='utf-8')
+
+            if len(cell_res_str) > max_result_length:
+                cell_res_str = cell_res_str[:max_result_length] + ' [...]'
+
+            self.grid.SetToolTipString(split_lines(cell_res_str))
+
     def get_visible_area(self):
         """Returns visible area
 
@@ -1194,12 +1194,26 @@ class SelectionActions(Actions):
                            changed=True)
 
         selection = self.get_selection()
+        current_table = self.grid.current_table
 
-        del_keys = \
-            [key for key in self.grid.code_array if key[:2] in selection]
+        for row, col, tab in self.grid.code_array.dict_grid.keys():
+            if tab == current_table and (row, col) in selection:
+                self.grid.actions.delete_cell((row, col, tab),
+                                              mark_unredo=False)
 
-        for key in del_keys:
-            self.grid.actions.delete_cell(key, mark_unredo=False)
+        self.grid.code_array.unredo.mark()
+
+        self.grid.code_array.result_cache.clear()
+
+    def quote_selection(self):
+        """Quotes selected cells, marks content as changed"""
+
+        selection = self.get_selection()
+        current_table = self.grid.current_table
+        for row, col, tab in self.grid.code_array.dict_grid.keys():
+            if tab == current_table and (row, col) in selection:
+                self.grid.actions.quote_code((row, col, tab),
+                                             mark_unredo=False)
 
         self.grid.code_array.unredo.mark()
 
