@@ -559,8 +559,13 @@ class ClipboardActions(Actions):
                        for line in data.split("\n")]
 
             except Exception, err:
-                error_msg(err)
-                return
+                # This must just be text.
+                try:
+                    obj = [line.split('\t') for line in data.split('\n')]
+                except Exception, err:
+                    # Now I really have no idea
+                    error_msg(err)
+                    return
 
         except ValueError, err:
             error_msg(err)
@@ -591,7 +596,11 @@ class MacroActions(Actions):
         post_command_event(self.main_window, self.ContentChangedMsg,
                            changed=True)
 
-        self.grid.code_array.execute_macros()
+        (result, err) = self.grid.code_array.execute_macros()
+
+        # Post event to macro dialog
+        post_command_event(self.main_window, self.MacroErrorMsg,
+                           msg=result, err=err)
 
     def open_macros(self, filepath):
         """Loads macros from file and marks grid as changed
@@ -604,7 +613,19 @@ class MacroActions(Actions):
         """
 
         try:
-            macro_infile = open(filepath, "r")
+            wx.BeginBusyCursor()
+            with open(filepath) as macro_infile:
+                # Enter safe mode ()
+                self.main_window.grid.actions.enter_safe_mode()
+                post_command_event(self.main_window, self.SafeModeEntryMsg)
+
+                # Mark content as changed
+                post_command_event(self.main_window, self.ContentChangedMsg,
+                                   changed=True)
+
+                macrocode = macro_infile.read()
+
+                self.grid.code_array.macros += "\n" + macrocode.strip("\n")
 
         except IOError:
             msg = _("Error opening file {filepath}.").format(filepath=filepath)
@@ -612,17 +633,8 @@ class MacroActions(Actions):
 
             return False
 
-        # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
-
-        macrocode = macro_infile.read()
-        macro_infile.close()
-
-        self.grid.code_array.macros += "\n" + macrocode.strip("\n")
-
-        self.main_window.grid.actions.enter_safe_mode()
-        post_command_event(self.main_window, self.SafeModeEntryMsg)
+        finally:
+            wx.EndBusyCursor()
 
     def save_macros(self, filepath, macros):
         """Saves macros to file
@@ -639,23 +651,35 @@ class MacroActions(Actions):
         io_error_text = _("Error writing to file {filepath}.")
         io_error_text = io_error_text.format(filepath=filepath)
 
+        # Make sure that old macro file does not get lost on abort save
+        tmpfile = filepath + "~"
+
         try:
-            with open(filepath, "w") as macro_outfile:
+            wx.BeginBusyCursor()
+            with open(tmpfile, "w") as macro_outfile:
                 macro_outfile.write(macros)
 
+            # Move save file from temp file to filepath
+            try:
+                os.rename(tmpfile, filepath)
+
+            except OSError:
+                # No tmp file present
+                pass
+
         except IOError:
-            txt = _("Error opening file {filepath}.").format(filepath=filepath)
             try:
                 post_command_event(self.main_window, self.StatusBarMsg,
-                                   text=txt)
+                                   text=io_error_text)
             except TypeError:
                 # The main window does not exist any more
                 pass
 
-            wx.EndBusyCursor()
-
             return False
 
+        finally:
+            wx.EndBusyCursor()
+
 
 class HelpActions(Actions):
     """Actions for getting help"""
