@@ -35,7 +35,7 @@ Provides:
   - DimensionsEntryDialog
   - CellEntryDialog
   - AboutDialog
-  - ConfigDialog
+  - PreferencesDialog
   - GPGParamsDialog
   - PasteAsDialog
 
@@ -61,7 +61,12 @@ from src.gui._events import post_command_event
 from src.gui._events import MainWindowEventMixin, GridEventMixin
 from src.lib.__csv import Digest, sniff, get_first_line, encode_gen
 from src.lib.__csv import csv_digest_gen, cell_key_val_gen
+from src.lib.exception_handling import get_user_codeframe
 
+import ast
+from traceback import print_exception
+from StringIO import StringIO
+from sys import exc_info
 #use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
 
@@ -546,8 +551,12 @@ class CSVPreviewGrid(wx.grid.Grid):
 
         # Fill in the rest of the lines
 
-        self.dtypes = \
-            [self.digest_types[key] for key in self.get_digest_keys()]
+        self.dtypes = []
+        for key in self.get_digest_keys():
+            try:
+                self.dtypes.append(self.digest_types[key])
+            except KeyError:
+                self.dtypes.append(types.NoneType)
 
         topleft = (has_header + 1, 0)
 
@@ -804,12 +813,12 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
             wx.TE_MULTILINE
         self.codetext_ctrl = PythonSTC(self.upper_panel, -1, style=style)
 
-        style = wx.TE_MULTILINE | wx.TE_READONLY
+        style = wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_RICH
         self.result_ctrl = wx.TextCtrl(self.lower_panel, -1, style=style)
 
         self.ok_button = wx.Button(self.lower_panel, wx.ID_OK)
         self.apply_button = wx.Button(self.lower_panel, wx.ID_APPLY)
-        self.cancel_button = wx.Button(self.lower_panel, wx.ID_CANCEL)
+        self.close_button = wx.Button(self.lower_panel, wx.ID_CLOSE)
 
         self._set_properties()
         self._do_layout()
@@ -820,7 +829,11 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
         self.Bind(stc.EVT_STC_MODIFIED, self.OnText, self.codetext_ctrl)
         self.Bind(wx.EVT_BUTTON, self.OnOk, self.ok_button)
         self.Bind(wx.EVT_BUTTON, self.OnApply, self.apply_button)
-        self.Bind(wx.EVT_BUTTON, self.OnCancel, self.cancel_button)
+        self.Bind(wx.EVT_BUTTON, self.OnClose, self.close_button)
+        parent.Bind(self.EVT_CMD_MACROERR, self.update_result_ctrl)
+
+        # States
+        self._ok_pressed = False
 
     def _do_layout(self):
         """Layout sizers"""
@@ -837,14 +850,14 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
         lower_sizer.Add(button_sizer, 1, wx.EXPAND, 0)
         button_sizer.Add(self.ok_button, 1, wx.EXPAND, 0)
         button_sizer.Add(self.apply_button, 1, wx.EXPAND, 0)
-        button_sizer.Add(self.cancel_button, 1, wx.EXPAND, 0)
+        button_sizer.Add(self.close_button, 1, wx.EXPAND, 0)
 
         self.upper_panel.SetSizer(upper_sizer)
         self.lower_panel.SetSizer(lower_sizer)
 
         self.splitter.SplitHorizontally(self.upper_panel,
                                         self.lower_panel,
-                                        500)
+                                        400)
         dialog_main_sizer.Add(self.splitter, 1, wx.EXPAND, 0)
         self.SetSizer(dialog_main_sizer)
         self.Layout()
@@ -857,7 +870,7 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
         self.codetext_ctrl.SetToolTipString(_("Enter python code here."))
         self.ok_button.SetToolTipString(_("Accept all changes"))
         self.apply_button.SetToolTipString(_("Apply changes to current macro"))
-        self.cancel_button.SetToolTipString(_("Remove current macro"))
+        self.close_button.SetToolTipString(_("Remove current macro"))
         self.splitter.SetBackgroundStyle(wx.BG_STYLE_COLOUR)
         self.result_ctrl.SetMinSize((10, 10))
 
@@ -869,24 +882,78 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
     def OnOk(self, event):
         """Event handler for Ok button"""
 
+        self._ok_pressed = True
         self.OnApply(event)
 
-        self.Destroy()
-
     def OnApply(self, event):
         """Event handler for Apply button"""
 
-        post_command_event(self.parent, self.MacroReplaceMsg,
-                           macros=self.macros)
-        post_command_event(self.parent, self.MacroExecuteMsg)
+        # See if we have valid python
+        try:
+            ast.parse(self.macros)
+        except:
+            # Grab the traceback and print it for the user
+            s = StringIO()
+            e = exc_info()
+            # usr_tb will more than likely be none because ast throws
+            #   SytnaxErrorsas occurring outside of the current
+            #   execution frame
+            usr_tb = get_user_codeframe(e[2]) or None
+            print_exception(e[0], e[1], usr_tb, None, s)
+            post_command_event(self.parent, self.MacroErrorMsg,
+                               err=s.getvalue())
+            success = False
+        else:
+            self.result_ctrl.SetValue('')
+            post_command_event(self.parent, self.MacroReplaceMsg,
+                               macros=self.macros)
+            post_command_event(self.parent, self.MacroExecuteMsg)
+            success = True
 
         event.Skip()
+        return success
 
-    def OnCancel(self, event):
+    def OnClose(self, event):
         """Event handler for Cancel button"""
 
+        # Warn if any unsaved changes
+        if self.parent.grid.code_array.macros != self.macros:
+            dlg = wx.MessageDialog(
+                self, _("There are changes in the macro editor "
+                        "which have not yet been applied.  Are you sure you "
+                        "wish to close the editor?"), _("Close Editor"),
+                wx.YES_NO | wx.ICON_WARNING)
+            if dlg.ShowModal() == wx.ID_NO:
+                return
+
         self.Destroy()
 
+    def update_result_ctrl(self, event):
+        """Update event result following execution by main window"""
+
+        # Check to see if macro window still exists
+        if not self:
+            return
+
+        printLen = 0
+        self.result_ctrl.SetValue('')
+        if hasattr(event, 'msg'):
+            # Output of script (from print statements, for example)
+            self.result_ctrl.AppendText(event.msg)
+            printLen = len(event.msg)
+        if hasattr(event, 'err'):
+            # Error messages
+            errLen = len(event.err)
+            errStyle = wx.TextAttr(wx.RED)
+            self.result_ctrl.AppendText(event.err)
+            self.result_ctrl.SetStyle(printLen, printLen+errLen, errStyle)
+
+        if not hasattr(event, 'err') or event.err == '':
+            # No error passed.  Close dialog if user requested it.
+            if self._ok_pressed:
+                self.Destroy()
+        self._ok_pressed = False
+
 
 # end of class MacroDialog
 
@@ -1203,6 +1270,13 @@ class PreferencesDialog(wx.Dialog):
             "widget_params": {"min": 0, "allow_long": True},
             "prepocessor": int,
         }),
+        ("timeout", {
+            "label": _(u"Timeout"),
+            "tooltip": _(u"Maximum time that an evaluation process may take."),
+            "widget": wx.lib.intctrl.IntCtrl,
+            "widget_params": {"min": 0, "allow_long": True},
+            "prepocessor": int,
+        }),
         ("gpg_key_uid", {
             "label": _(u"GPG key name"),
             "tooltip": _(u"Name of the GPG key for signing files"),
@@ -1377,4 +1451,4 @@ class PasteAsDialog(wx.Dialog):
             "transpose": self.transpose_checkbox.GetValue()
         }
 
-    parameters = property(get_parameters)
\ No newline at end of file
+    parameters = property(get_parameters)
