@@ -85,6 +85,7 @@ class MainWindowEventMixin(object):
     MacroExecuteMsg, EVT_CMD_MACROEXECUTE = new_command_event()
     MacroLoadMsg, EVT_CMD_MACROLOAD = new_command_event()
     MacroSaveMsg, EVT_CMD_MACROSAVE = new_command_event()
+    MacroErrorMsg, EVT_CMD_MACROERR = new_command_event()
 
     MainToolbarToggleMsg, EVT_CMD_MAINTOOLBAR_TOGGLE = new_command_event()
     MacroToolbarToggleMsg, EVT_CMD_MACROTOOLBAR_TOGGLE = new_command_event()
@@ -166,15 +167,17 @@ class GridEventMixin(object):
 
     # Grid edit mode events
 
+    SelectAll, EVT_CMD_SELECT_ALL = new_command_event()
     EnterSelectionModeMsg, EVT_CMD_ENTER_SELECTION_MODE = new_command_event()
     ExitSelectionModeMsg, EVT_CMD_EXIT_SELECTION_MODE = new_command_event()
     SelectionMsg, EVT_CMD_SELECTION = new_command_event()
 
     # Grid view events
 
+    ViewFrozenMsg, EVT_CMD_VIEW_FROZEN = new_command_event()
     RefreshSelectionMsg, EVT_CMD_REFRESH_SELECTION = new_command_event()
     DisplayGotoCellDialogMsg, EVT_CMD_DISPLAY_GOTO_CELL_DIALOG = \
-                                                        new_command_event()
+        new_command_event()
     GotoCellMsg, EVT_CMD_GOTO_CELL = new_command_event()
     ZoomInMsg, EVT_CMD_ZOOM_IN = new_command_event()
     ZoomOutMsg, EVT_CMD_ZOOM_OUT = new_command_event()
@@ -196,7 +199,9 @@ class GridEventMixin(object):
     DeleteTabsMsg, EVT_CMD_DELETE_TABS = new_command_event()
 
     ShowResizeGridDialogMsg, EVT_CMD_SHOW_RESIZE_GRID_DIALOG = \
-                                                    new_command_event()
+        new_command_event()
+
+    QuoteMsg, EVT_CMD_QUOTE = new_command_event()
 
     TableChangedMsg, EVT_CMD_TABLE_CHANGED = new_command_event()
 
