@@ -65,6 +65,12 @@ class GridRenderer(wx.grid.PyGridCellRenderer):
         # bordercolor_bottom, bordercolor_right)
         self.backgrounds = {}
 
+        # Fontcache speeds up font retrieval
+        self.font_cache = {}
+
+        # Bitmap cache speeds up bitmap scaling
+        self.bmp_cache = {}
+
         # Zoom of grid
         self.zoom = 1.0
 
@@ -142,73 +148,6 @@ class GridRenderer(wx.grid.PyGridCellRenderer):
         dc.DrawLine(pt_lr[0], pt_lr[1], pt_ur[0], pt_ur[1])
         dc.DrawLine(pt_ur[0], pt_ur[1], pt_ul[0], pt_ul[1])
 
-    def _get_full_cells(self, dc, grid, key, text_pos, text_extent):
-        """Generator of full cells from key in direction
-
-        Parameters
-        ----------
-        key: 3-tuple of Integer
-        \tCurrent cell
-        text_pos: 3-tuple
-        \tPosition and direction of text
-
-        """
-
-        row, col, tab = key
-
-        blocking_distance = None
-
-        textbox = self.get_text_rotorect(text_pos, text_extent)
-
-        for distance, __row, __col in grid.colliding_cells(row, col, textbox):
-            # Draw blocking arrows if locking cell is not empty
-
-            if not(
-               (blocking_distance is None or distance == blocking_distance)
-               and not self.data_array[__row, __col, tab]):
-
-                yield __row, __col, tab
-
-    def _get_empty_cells(self, dc, grid, key, text_pos, text_extent):
-        """Generator of empty cells from key in direction
-
-        Parameters
-        ----------
-        key: 3-tuple of Integer
-        \tCurrent cell
-        text_pos: 3-tuple
-        \tPosition and direction of text
-
-        """
-
-        row, col, tab = key
-
-        blocking_distance = None
-
-        textbox = self.get_text_rotorect(text_pos, text_extent)
-
-        for distance, __row, __col in grid.colliding_cells(row, col, textbox):
-
-            if blocking_distance is None or distance == blocking_distance:
-                if self.data_array[__row, __col, tab] is not None and \
-                   self.data_array[__row, __col, tab] != "":
-                    blocking_distance = distance
-                else:
-                    yield __row, __col, tab
-
-    def _get_available_space_rects(self, dc, grid, key, rect, text_pos,
-                                   text_extent, res_text):
-        """Returns rects needed by key cell that are in available space"""
-
-        yield rect
-
-        for cell in self._get_empty_cells(dc, grid, key,
-                                          text_pos, text_extent):
-            __row, __col, _ = cell
-            cell_rect = grid.CellToRect(__row, __col)
-
-            yield cell_rect
-
     def draw_text_label(self, dc, res, rect, grid, key):
         """Draws text label of cell
 
@@ -254,13 +193,12 @@ class GridRenderer(wx.grid.PyGridCellRenderer):
 
         font = self.get_font(textfont, pointsize, fontweight, fontstyle,
                              underline)
-        font.SetFaceName(textfont)  # Windows hack
         dc.SetFont(font)
 
         text_x, text_y = self.get_text_position(dc, rect, res_text, angle,
                                                 vertical_align, justification)
 
-        __rect = xrect.Rect(rect.x, rect.y, rect.width, rect.height)
+        #__rect = xrect.Rect(rect.x, rect.y, rect.width, rect.height)
 
         text_extent = dc.GetTextExtent(res_text)
 
@@ -271,30 +209,13 @@ class GridRenderer(wx.grid.PyGridCellRenderer):
 
         text_pos = text_x, text_y, angle
 
-        if all(__rect.is_point_in_rect(*textedge) for textedge in
-               self.get_textbox_edges(text_pos, text_extent)):
-            clipping = False
-        else:
-            clipping = True
-            clip_rects = \
-                self._get_available_space_rects(dc, grid, key, rect, text_pos,
-                                                text_extent, res_text)
-
-        if clipping:
-            for clip_rect in clip_rects:
-                dc.SetClippingRect(clip_rect)
-                dc.DrawRotatedText(res_text, *text_pos)
-                text_extent = dc.GetTextExtent(res_text)
-                if strikethrough:
-                    self._draw_strikethrough_line(grid, dc, rect, text_x,
-                                                  text_y, angle, text_extent)
-                dc.DestroyClippingRegion()
-        else:
-            dc.DrawRotatedText(res_text, *text_pos)
-            text_extent = dc.GetTextExtent(res_text)
-            if strikethrough:
-                self._draw_strikethrough_line(grid, dc, rect, text_x, text_y,
-                                              angle, text_extent)
+        dc.SetClippingRect(rect)
+        dc.DrawRotatedText(res_text, *text_pos)
+        text_extent = dc.GetTextExtent(res_text)
+        if strikethrough:
+            self._draw_strikethrough_line(grid, dc, rect, text_x,
+                                          text_y, angle, text_extent)
+        dc.DestroyClippingRegion()
 
     def _draw_strikethrough_line(self, grid, dc, rect,
                                  string_x, string_y, angle, text_extent):
@@ -350,13 +271,22 @@ class GridRenderer(wx.grid.PyGridCellRenderer):
 
         """
 
+        fontsize = self.get_zoomed_size(pointsize)
+
+        font_key = textfont, fontsize, fontweight, fontstyle, underline
+        if font_key in self.font_cache:
+            return self.font_cache[font_key]
+
         # Get a real font from textfont string
 
         font = get_font_from_data(textfont)
-        font.SetPointSize(self.get_zoomed_size(pointsize))
+        font.SetPointSize(fontsize)
         font.SetWeight(fontweight)
         font.SetStyle(fontstyle)
         font.SetUnderlined(underline)
+        font.SetFaceName(textfont)  # Windows hack
+
+        self.font_cache[font_key] = font
 
         return font
 
@@ -510,10 +440,6 @@ class GridRenderer(wx.grid.PyGridCellRenderer):
 
                 rect = wx.Rect(ul_rect.x, ul_rect.y, width, height)
 
-                # Adjust rect to 1 pixel error
-                rect.width -= 1
-                rect.height -= 1
-
                 return rect
 
     def draw_bitmap(self, dc, bmp, rect, grid, key, scale=True):
@@ -523,15 +449,22 @@ class GridRenderer(wx.grid.PyGridCellRenderer):
 
         """
 
-        def scale(bmp, width, height):
+        def scale(img, width, height):
             """Returns a scaled version of the bitmap bmp"""
 
-            img = bmp.ConvertToImage()
             img = img.Scale(width, height, quality=wx.IMAGE_QUALITY_HIGH)
             return wx.BitmapFromImage(img)
 
+
         if scale:
-            bmp = scale(bmp, rect.width, rect.height)
+            img = bmp.ConvertToImage()
+            bmp_key = img, rect.width, rect.height
+
+            if bmp_key in self.bmp_cache:
+                return self.bmp_cache[bmp_key]
+
+            bmp = scale(*bmp_key)
+            self.bmp_cache[bmp_key] = bmp
 
         dc.DrawBitmap(bmp, rect.x, rect.y)
 
@@ -578,6 +511,8 @@ class GridRenderer(wx.grid.PyGridCellRenderer):
             bg_components = ["bgcolor",
                              "borderwidth_bottom", "borderwidth_right",
                              "bordercolor_bottom", "bordercolor_right"]
+            if grid._view_frozen:
+                bg_components += ['frozen']
 
             bg_key = tuple([width, height] +
                            [self.data_array.cell_attributes[key][bgc]
@@ -595,13 +530,8 @@ class GridRenderer(wx.grid.PyGridCellRenderer):
                 bg = self.backgrounds[bg_key] = \
                     Background(grid, rect, self.data_array, *key)
 
-        if is_gtk() and not printing:
-            mask_type = wx.AND
-        else:
-            mask_type = wx.COPY
-
         dc.Blit(rect.x, rect.y, rect.width, rect.height,
-                bg.dc, 0, 0, mask_type)
+                bg.dc, 0, 0, wx.COPY)
 
         # Check if the dc is drawn manually be a return func
         try:
@@ -666,19 +596,27 @@ class Background(object):
     def draw_background(self, dc):
         """Draws the background of the background"""
 
+        attr = self.data_array.cell_attributes[self.key]
+
         if self.selection:
             color = get_color(config["selection_color"])
         else:
-            rgb = self.data_array.cell_attributes[self.key]["bgcolor"]
+            rgb = attr["bgcolor"]
             color = wx.Colour()
             color.SetRGB(rgb)
-
         bgbrush = wx.Brush(color, wx.SOLID)
-
         dc.SetBrush(bgbrush)
         dc.SetPen(wx.TRANSPARENT_PEN)
         dc.DrawRectangle(0, 0, self.rect.width, self.rect.height)
 
+        # Draw frozen cell background rect
+        if self.grid._view_frozen and attr['frozen']:
+            style = wx.FDIAGONAL_HATCH
+            freeze_color = get_color(config['freeze_color'])
+            freeze_brush = wx.Brush(freeze_color, style)
+            dc.SetBrush(freeze_brush)
+            dc.DrawRectangle(0, 0, self.rect.width, self.rect.height)
+
     def draw_border_lines(self, dc):
         """Draws lines"""
 
@@ -695,18 +633,30 @@ class Background(object):
 
         # Bottom line pen
 
-        color = cell_attributes[key]["bordercolor_bottom"]
-        width = cell_attributes[key]["borderwidth_bottom"]
-        bottom_pen = get_pen_from_data((color, width, int(wx.SOLID)))
+        bottom_color = cell_attributes[key]["bordercolor_bottom"]
+        bottom_width = cell_attributes[key]["borderwidth_bottom"]
+        bottom_pen = get_pen_from_data(
+            (bottom_color, bottom_width, int(wx.SOLID)))
 
         # Right line pen
 
-        color = cell_attributes[key]["bordercolor_right"]
-        width = cell_attributes[key]["borderwidth_right"]
-        right_pen = get_pen_from_data((color, width, int(wx.SOLID)))
+        right_color = cell_attributes[key]["bordercolor_right"]
+        right_width = cell_attributes[key]["borderwidth_right"]
+        right_pen = get_pen_from_data(
+            (right_color, right_width, int(wx.SOLID)))
 
         borderpens = [bottom_pen, right_pen]
 
+        # If 0 width then no border is drawn
+
+        if bottom_width == 0:
+            borderpens.pop(0)
+            lines.pop(0)
+
+        if right_width == 0:
+            borderpens.pop(-1)
+            lines.pop(-1)
+
         # Topmost line if in topmost cell
 
         if row == 0:
