@@ -304,6 +304,9 @@ class MainWindow(wx.Frame, EventMixin):
         self.Bind(self.EVT_CMD_PRINT_PREVIEW, handlers.OnPrintPreview)
         self.Bind(self.EVT_CMD_PRINT, handlers.OnPrint)
 
+        # Edit Events
+        self.Bind(self.EVT_CMD_SELECT_ALL, handlers.OnSelectAll)
+
         # Clipboard events
 
         self.Bind(self.EVT_CMD_CUT, handlers.OnCut)
@@ -654,10 +657,20 @@ class MainWindowEventHandlers(EventMixin):
 
         # Get filepath from user
 
-        wildcard = \
-            _("Pyspread file") + " (*.pys)|*.pys|" + \
-            _("All files") + " (*.*)|*.*"
-        message = _("Choose pyspread file to open.")
+        try:
+            import xlrd
+            wildcard = \
+                _("Pyspread file") + " (*.pys)|*.pys|" + \
+                _("Excel file") + " (*.xls)|*.xls|" + \
+                _("All files") + " (*.*)|*.*"
+
+        except ImportError:
+            wildcard = \
+                _("Pyspread file") + " (*.pys)|*.pys|" + \
+                _("All files") + " (*.*)|*.*"
+            xlrd = None
+
+        message = _("Choose file to open.")
         style = wx.OPEN
         filepath, filterindex = \
             self.interfaces.get_filepath_findex_from_user(wildcard, message,
@@ -666,6 +679,8 @@ class MainWindowEventHandlers(EventMixin):
         if filepath is None:
             return
 
+        filetype = "pys" if xlrd is None or filterindex != 1 else "xls"
+
         # Change the main window filepath state
 
         self.main_window.filepath = filepath
@@ -673,7 +688,7 @@ class MainWindowEventHandlers(EventMixin):
         # Load file into grid
         post_command_event(self.main_window,
                            self.main_window.GridActionOpenMsg,
-                           attr={"filepath": filepath})
+                           attr={"filepath": filepath, "filetype": filetype})
 
         # Set Window title to new filepath
 
@@ -965,10 +980,18 @@ class MainWindowEventHandlers(EventMixin):
     def OnCut(self, event):
         """Clipboard cut event handler"""
 
-        data = self.main_window.actions.cut(self.main_window.grid.selection)
-        self.main_window.clipboard.set_clipboard(data)
+        entry_line = \
+            self.main_window.entry_line_panel.entry_line_panel.entry_line
 
-        self.main_window.grid.ForceRefresh()
+        if wx.Window.FindFocus() != entry_line:
+            selection = self.main_window.grid.selection
+            data = self.main_window.actions.cut(selection)
+            self.main_window.clipboard.set_clipboard(data)
+
+            self.main_window.grid.ForceRefresh()
+
+        else:
+            entry_line.Cut()
 
         event.Skip()
 
@@ -1039,6 +1062,19 @@ class MainWindowEventHandlers(EventMixin):
 
         event.Skip()
 
+    # Edit events
+    def OnSelectAll(self, event):
+        """Select all cells event handler"""
+
+        entry_line = \
+            self.main_window.entry_line_panel.entry_line_panel.entry_line
+
+        if wx.Window.FindFocus() != entry_line:
+            self.main_window.grid.SelectAll()
+
+        else:
+            entry_line.SelectAll()
+
     # View events
 
     def OnFocusFind(self, event):
