@@ -44,7 +44,7 @@ from src.lib.testlib import params, pytest_generate_tests
 from src.lib.parsers import get_font_from_data, get_pen_from_data
 
 param_font = [
-    {"fontdata": "Courier 13", "face": "Courier", "size": 13},
+    {"fontdata": "Courier New 13", "face": "Courier New", "size": 13},
     {"fontdata": "Arial 43", "face": "Arial", "size": 43},
 ]
 
