@@ -197,206 +197,7 @@ class CellAttributes(list):
 # End of class CellAttributes
 
 
-class ParserMixin(object):
-    """Provides parser methods for DictGrid"""
-
-    def _split_tidy(self, string, maxsplit=None):
-        """Rstrips string for \n and splits string for \t"""
-
-        if maxsplit is None:
-            return string.rstrip("\n").split("\t")
-        else:
-            return string.rstrip("\n").split("\t", maxsplit)
-
-    def _get_key(self, *keystrings):
-        """Returns int key tuple from key string list"""
-
-        return tuple(imap(int, keystrings))
-
-    def parse_to_shape(self, line):
-        """Parses line and adjusts grid shape"""
-
-        self.shape = self._get_key(*self._split_tidy(line))
-
-    def parse_to_grid(self, line):
-        """Parses line and inserts grid data"""
-
-        row, col, tab, code = self._split_tidy(line, maxsplit=3)
-        key = self._get_key(row, col, tab)
-
-        self[key] = unicode(code, encoding='utf-8')
-
-    def parse_to_attribute(self, line):
-        """Parses line and appends cell attribute"""
-
-        splitline = self._split_tidy(line)
-
-        selection_data = map(ast.literal_eval, splitline[:5])
-        selection = Selection(*selection_data)
-
-        tab = int(splitline[5])
-
-        attrs = {}
-        for col, ele in enumerate(splitline[6:]):
-            if not (col % 2):
-                # Odd entries are keys
-                key = ast.literal_eval(ele)
-
-            else:
-                # Even cols are values
-                attrs[key] = ast.literal_eval(ele)
-
-        self.cell_attributes.append((selection, tab, attrs))
-
-    def parse_to_height(self, line):
-        """Parses line and inserts row hight"""
-
-        # Split with maxsplit 3
-        row, tab, height = self._split_tidy(line)
-        key = self._get_key(row, tab)
-
-        try:
-            self.row_heights[key] = float(height)
-
-        except ValueError:
-            pass
-
-    def parse_to_width(self, line):
-        """Parses line and inserts column width"""
-
-        # Split with maxsplit 3
-        col, tab, width = self._split_tidy(line)
-        key = self._get_key(col, tab)
-
-        try:
-            self.col_widths[key] = float(width)
-
-        except ValueError:
-            pass
-
-    def parse_to_macro(self, line):
-        """Appends line to macro"""
-
-        self.macros += line
-
-# End of class ParserMixin
-
-
-class StringGeneratorMixin(object):
-    """String generation methods for DictGrid"""
-
-    def grid_to_strings(self):
-        """Yields a string that represents the grid content for saving
-
-        Format
-        ------
-        [shape]
-        rows\tcols\ttabs\n
-        [grid]
-        row\tcol\ttab\tcode\n
-        row\tcol\ttab\tcode\n
-        ...
-
-        """
-
-        yield u"[shape]\n"
-        yield u"\t".join(map(unicode, self.shape)) + u"\n"
-
-        yield u"[grid]\n"
-
-        for key in self:
-            key_str = u"\t".join(repr(ele) for ele in key)
-            code_str = unicode(self[key])
-
-            yield key_str + u"\t" + code_str + u"\n"
-
-    def attributes_to_strings(self):
-        """Yields a string that represents the cell attributes for saving
-
-        Format
-        ------
-
-        [attributes]
-        selection[0]\t...\tselection[5]\ttab\tkey\tvalue\t...\tkey\tvalue\n
-        ...
-
-        """
-
-        yield u"[attributes]\n"
-
-        for selection, tab, attr_dict in self.cell_attributes:
-            sel_list = [selection.block_tl, selection.block_br,
-                        selection.rows, selection.cols, selection.cells]
-
-            tab_list = [tab]
-
-            attr_dict_list = []
-            for key in attr_dict:
-                attr_dict_list.append(key)
-                attr_dict_list.append(attr_dict[key])
-
-            line_list = map(repr, sel_list + tab_list + attr_dict_list)
-
-            yield u"\t".join(line_list) + u"\n"
-
-    def heights_to_strings(self):
-        """Yields a string that represents the row heights for saving
-
-        Format
-        ------
-
-        [row_heights]
-        row\ttab\tvalue\n
-        ...
-
-        """
-
-        yield u"[row_heights]\n"
-
-        for row, tab in self.row_heights:
-            height = self.row_heights[(row, tab)]
-            height_strings = map(repr, [row, tab, height])
-            yield u"\t".join(height_strings) + u"\n"
-
-    def widths_to_strings(self):
-        """Yields a string that represents the column widths for saving
-
-        Format
-        ------
-
-        [col_widths]
-        col\ttab\tvalue\n
-        ...
-
-        """
-
-        yield u"[col_widths]\n"
-
-        for col, tab in self.col_widths:
-            width = self.col_widths[(col, tab)]
-            width_strings = map(repr, [col, tab, width])
-            yield u"\t".join(width_strings) + u"\n"
-
-    def macros_to_strings(self):
-        """Yields a string that represents the content for saving
-
-        Format
-        ------
-
-        [macros]
-        Macro code
-
-        """
-
-        yield u"[macros]\n"
-
-        for line in self.macros.split("\n"):
-            yield line + u"\n"
-
-# End of class StringGeneratorMixin
-
-
-class DictGrid(KeyValueStore, ParserMixin, StringGeneratorMixin):
+class DictGrid(KeyValueStore):
     """The core data class with all information that is stored in a pys file.
 
     Besides grid code access via standard dict operations, it provides
@@ -470,28 +271,132 @@ class DataArray(object):
         # Safe mode
         self.safe_mode = False
 
+    # Data is the central content interface for loading / saving data.
+    # It shall be used for loading and saving from and to pys and other files.
+    # It shall be used for loading and saving macros.
+    # It is not used for importinf and exporting data because these operations
+    # are partial to the grid.
+
+    def _get_data(self):
+        """Returns dict of data content.
+
+        Keys
+        ----
+
+        shape: 3-tuple of Integer
+        \tGrid shape
+        grid: Dict of 3-tuples to strings
+        \tCell content
+        attributes: List of 3-tuples
+        \tCell attributes
+        row_heights: Dict of 2-tuples to float
+        \t(row, tab): row_height
+        col_widths: Dict of 2-tuples to float
+        \t(col, tab): col_width
+        macros: String
+        \tMacros from macro list
+
+        """
+
+        data = {}
+
+        data["shape"] = self.shape
+        data["grid"] = {}.update(self.dict_grid)
+        data["attributes"] = [ca for ca in self.cell_attributes]
+        data["row_heights"] = self.row_heights
+        data["col_widths"] = self.col_widths
+        data["macros"] = self.macros
+
+        return data
+
+    def _set_data(self, **kwargs):
+        """Sets data from given parameters
+
+        Old values are deleted.
+        If a paremeter is not given, nothing is changed.
+
+        Parameters
+        ----------
+
+        shape: 3-tuple of Integer
+        \tGrid shape
+        grid: Dict of 3-tuples to strings
+        \tCell content
+        attributes: List of 3-tuples
+        \tCell attributes
+        row_heights: Dict of 2-tuples to float
+        \t(row, tab): row_height
+        col_widths: Dict of 2-tuples to float
+        \t(col, tab): col_width
+        macros: String
+        \tMacros from macro list
+
+        """
+
+        if "shape" in kwargs:
+            self.shape = kwargs["shape"]
+
+        if "grid" in kwargs:
+            self.dict_grid.clear()
+            self.dict_grid.update(kwargs["grid"])
+
+        if "attributes" in kwargs:
+            self.attributes[:] = kwargs["attributes"]
+
+        if "row_heights" in kwargs:
+            self.row_heights = kwargs["row_heights"]
+
+        if "col_widths" in kwargs:
+            self.col_widths = kwargs["col_widths"]
+
+        if "macros" in kwargs:
+            self.macros = kwargs["macros"]
+
+    data = property(_get_data, _set_data)
+
     # Row and column attributes mask
     # Keys have the format (row, table)
 
-    @property
-    def row_heights(self):
+    def _get_row_heights(self):
         """Returns row_heights dict"""
 
         return self.dict_grid.row_heights
 
-    @property
-    def col_widths(self):
+    def _set_row_heights(self, row_heights):
+        """Sets  macros string"""
+
+        self.dict_grid.row_heights = row_heights
+
+    row_heights = property(_get_row_heights, _set_row_heights)
+
+    def _get_col_widths(self):
         """Returns col_widths dict"""
 
         return self.dict_grid.col_widths
 
+    def _set_col_widths(self, col_widths):
+        """Sets  macros string"""
+
+        self.dict_grid.col_widths = col_widths
+
+    col_widths = property(_get_col_widths, _set_col_widths)
+
     # Cell attributes mask
-    @property
-    def cell_attributes(self):
+    def _get_cell_attributes(self):
         """Returns cell_attributes list"""
 
         return self.dict_grid.cell_attributes
 
+    def _set_cell_attributes(self, value):
+        """Setter for cell_atributes"""
+
+        # Empty cell_attributes first
+        self.cell_attributes[:] = []
+        self.cell_attributes.extend(value)
+
+    cell_attributes = attributes = \
+        property(_get_cell_attributes, _set_cell_attributes)
+
     def __iter__(self):
         """Returns iterator over self.dict_grid"""
 
@@ -768,13 +673,6 @@ class DataArray(object):
 
                 break
 
-    def _set_cell_attributes(self, value):
-        """Setter for cell_atributes"""
-
-        # Empty cell_attributes first
-        self.cell_attributes[:] = []
-        self.cell_attributes.extend(value)
-
     def _shift_rowcol(self, insertion_point, no_to_insert, mark_unredo):
         """Shifts row and column sizes when a table is inserted or deleted"""
 
@@ -1276,6 +1174,17 @@ class CodeArray(DataArray):
             result = assignment_target_error
 
         else:
+
+            try:
+                import signal
+
+                signal.signal(signal.SIGALRM, self.handler)
+                signal.alarm(config["timeout"])
+
+            except:
+                # No POSIX system
+                pass
+
             try:
                 result = eval(expression, env, {})
 
@@ -1283,9 +1192,19 @@ class CodeArray(DataArray):
                 # Attribute Error includes RunTimeError
                 result = AttributeError(err)
 
+            except RuntimeError, err:
+                result = RuntimeError(err)
+
             except Exception, err:
                 result = Exception(err)
 
+            finally:
+                try:
+                    signal.alarm(0)
+                except:
+                    # No POSIX system
+                    pass
+
         # Change back cell value for evaluation from other cells
         self.dict_grid[key] = _old_code
 
@@ -1327,8 +1246,7 @@ class CodeArray(DataArray):
         """Clears all newly assigned globals"""
 
         base_keys = ['cStringIO', 'IntType', 'KeyValueStore', 'UnRedo',
-                     'is_generator_like', 'StringGeneratorMixin',
-                     'is_string_like', 'ParserMixin', 'bz2', 'base64',
+                     'is_generator_like', 'is_string_like', 'bz2', 'base64',
                      '__package__', 're', 'config', '__doc__', 'SliceType',
                      'CellAttributes', 'product', 'ast', '__builtins__',
                      '__file__', 'charts', 'sys', 'is_slice_like', '__name__',
@@ -1347,7 +1265,7 @@ class CodeArray(DataArray):
         """
 
         if self.safe_mode:
-            return "Safe mode activated. Code not executed."
+            return '', "Safe mode activated. Code not executed."
 
         # Windows exec does not like Windows newline
         self.macros = self.macros.replace('\r\n', '\n')
@@ -1358,22 +1276,45 @@ class CodeArray(DataArray):
         # Create file-like string to capture output
         code_out = cStringIO.StringIO()
         code_err = cStringIO.StringIO()
+        err_msg = cStringIO.StringIO()
 
         # Capture output and errors
         sys.stdout = code_out
         sys.stderr = code_err
 
+        try:
+            import signal
+
+            signal.signal(signal.SIGALRM, self.handler)
+            signal.alarm(config["timeout"])
+
+        except:
+            # No POSIX system
+            pass
+
         try:
             exec(self.macros, globals())
+            try:
+                signal.alarm(0)
+            except:
+                # No POSIX system
+                pass
 
-        except Exception, err:
-            print err
-
+        except Exception:
+            # Print exception
+            # (Because of how the globals are handled during execution
+            # we must import modules here)
+            from traceback import print_exception
+            from src.lib.exception_handling import get_user_codeframe
+            exc_info = sys.exc_info()
+            user_tb = get_user_codeframe(exc_info[2]) or exc_info[2]
+            print_exception(exc_info[0], exc_info[1], user_tb, None, err_msg)
         # Restore stdout and stderr
         sys.stdout = sys.__stdout__
         sys.stderr = sys.__stderr__
 
-        outstring = code_out.getvalue() + code_err.getvalue()
+        results = code_out.getvalue()
+        errs = code_err.getvalue() + err_msg.getvalue()
 
         code_out.close()
         code_err.close()
@@ -1384,7 +1325,7 @@ class CodeArray(DataArray):
         # Reset frozen cache
         self.frozen_cache.clear()
 
-        return outstring
+        return results, errs
 
     def _sorted_keys(self, keys, startkey, reverse=False):
         """Generator that yields sorted keys starting with startkey
@@ -1499,4 +1440,7 @@ class CodeArray(DataArray):
             if is_matching(key, find_string, flags):
                 return key
 
+    def handler(self, signum, frame):
+        raise RuntimeError("Timeout after {} s.".format(config["timeout"]))
+
 # End of class CodeArray
